//@line 36 "/builds/slave/rel-m-beta-lnx-mb-rpk-4/mozilla-beta/mobile/locales/en-US/mobile-l10n.js"

//@line 38 "/builds/slave/rel-m-beta-lnx-mb-rpk-4/mozilla-beta/mobile/locales/en-US/mobile-l10n.js"

pref("general.useragent.locale", "it");
